@@ -1,12 +1,11 @@
-__VERSION__ = "1.1.6"
-__CONDA__ = "celescope_RD"
+__VERSION__ = "1.1.8"
+__version__ = __VERSION__
 
 ASSAY_DICT = {
     "rna": "Single Cell RNA-Seq",
     "rna_virus": "Single Cell RNA-Seq Virus",
     'capture_virus': "Single Cell Capture Virus",
     'fusion': "Single Cell Fusion Gene",
-    'smk': "Single Cell SMK",
     'vdj': "Single Cell VDJ",
     'mut': 'Single Cell Insertion and Deletion',
     'hla': 'Single Cell HLA',
