@@ -1,16 +1,17 @@
 import os
-from celescope.tools.utils import format_number, log, STAR_util
+from celescope.tools.utils import *
 
 
-@log
+@add_log
 def mapping_mut(args):
     STAR_util(
         args.sample,
         args.outdir,
-        args.input_read,
+        args.fq,
         args.indel_genomeDir,
         args.thread,
         args.outFilterMatchNmin,
+        outBAMsortingBinsN=200,
     )
 
 
