@@ -1,6 +1,3 @@
-#!/bin/env python
-# coding=utf8
-
 import os
 import re
 import sys
@@ -9,13 +6,15 @@ import logging
 import subprocess
 import glob
 import pandas as pd
-from celescope.tools.utils import format_number, log, format_stat
-from celescope.tools.utils import glob_genomeDir
+from celescope.tools.utils import *
 from celescope.tools.report import reporter
 
 class Step_mapping():
 
-    def __init__(self, sample, outdir, assay, thread, fq, genomeDir, out_unmapped=False, debug=False, outFilterMatchNmin=0):
+    def __init__(self, sample, outdir, assay, thread, fq, genomeDir, 
+    out_unmapped=False, debug=False, outFilterMatchNmin=0, STAR_param="", sort_BAM=True,
+    outFilterMultimapNmax=1, STAR_index=None, refFlat=None, consensus_fq=False
+    ):
         self.sample = sample
         self.outdir = outdir
         self.assay = assay
@@ -25,6 +24,27 @@ class Step_mapping():
         self.out_unmapped = out_unmapped
         self.debug = debug
         self.outFilterMatchNmin = outFilterMatchNmin
+        self.STAR_param = STAR_param
+        self.sort_BAM = sort_BAM
+        self.multi_max = outFilterMultimapNmax
+        if self.genomeDir and self.genomeDir != "None":
+            self.STAR_index = self.genomeDir
+        else:
+            self.STAR_index = STAR_index
+            self.refFlat = refFlat
+        self.consensus_fq = consensus_fq
+
+        # set param
+        self.outPrefix = f'{self.outdir}/{self.sample}_'
+        self.STAR_map_log = f'{self.outdir}/{self.sample}_Log.final.out'
+        self.STAR_bam = f'{self.outdir}/{self.sample}_Aligned.sortedByCoord.out.bam'
+        if self.sort_BAM:
+            self.sort_suffix = 'SortedByCoordinate'
+        else:
+            self.sort_suffix = 'Unsorted'
+            self.unsort_STAR_bam = f'{self.outdir}/{self.sample}_Aligned.out.bam'
+
+
         if not os.path.exists(outdir):
             os.system('mkdir -p %s' % (outdir))
         self.stats = pd.Series()
@@ -32,6 +52,11 @@ class Step_mapping():
         self.step_name = 'STAR'
 
     def format_stat(self):
+
+        stat_prefix = 'Reads'
+        if self.consensus_fq:
+            stat_prefix = 'UMIs'
+
         fh1 = open(self.STAR_map_log, 'r')
         UNIQUE_READS = []
         MULTI_MAPPING_READS = []
@@ -72,11 +97,11 @@ class Step_mapping():
 
         self.stats = self.stats.append(pd.Series(
             f'{format_number(int(UNIQUE_READS[0]))}({UNIQUE_READS[1]})',
-            index=['Uniquely Mapped Reads']
+            index=[f'Uniquely Mapped {stat_prefix}']
         ))
         self.stats = self.stats.append(pd.Series(
             f'{format_number(int(MULTI_MAPPING_READS[0]))}({MULTI_MAPPING_READS[1]})',
-            index=['Multi-Mapped Reads']
+            index=[f'Multi-Mapped {stat_prefix}']
         ))
         # ribo
         if self.debug:
@@ -91,7 +116,7 @@ class Step_mapping():
 
             self.stats = self.stats.append(pd.Series(
                 format_stat(Reads_Mapped_to_rRNA, Reads_Total),
-                index=['Reads Mapped to rRNA']
+                index=[f'{stat_prefix} Mapped to rRNA']
             ))
             f.close()
 
@@ -113,7 +138,7 @@ class Step_mapping():
 
         self.stats.to_csv(self.stats_file, sep=':', header=False)
 
-    @log
+    @add_log
     def ribo(self):
         import celescope
         root_path = os.path.dirname(celescope.__file__)
@@ -131,23 +156,25 @@ class Step_mapping():
         Step_mapping.ribo.logger.info(cmd)
         subprocess.check_call(cmd, shell=True)
     
-    @log
+    @add_log
     def STAR(self):
-        self.outPrefix = f'{self.outdir}/{self.sample}_'
-        self.STAR_bam = f'{self.outdir}/{self.sample}_Aligned.sortedByCoord.out.bam'
-        self.STAR_map_log = f'{self.outdir}/{self.sample}_Log.final.out'    
-        cmd = ['STAR', '--runThreadN', str(self.thread), '--genomeDir', self.genomeDir,
-            '--readFilesIn', self.fq, '--readFilesCommand', 'zcat', '--outFilterMultimapNmax',
-            '1', '--outFileNamePrefix', self.outPrefix, '--outSAMtype', 'BAM', 'SortedByCoordinate',
+        cmd = ['STAR', '--runThreadN', str(self.thread), '--genomeDir', self.STAR_index,
+            '--readFilesIn', self.fq, '--outFilterMultimapNmax', str(self.multi_max), 
+            '--outFileNamePrefix', self.outPrefix, '--outSAMtype', 'BAM', self.sort_suffix,
             '--outFilterMatchNmin', str(self.outFilterMatchNmin)]
         if self.out_unmapped:
             cmd += ['--outReadsUnmapped', 'Fastx']
-        Step_mapping.STAR.logger.info(' '.join(cmd))
-        subprocess.check_call(cmd)
+        if self.fq[len(self.fq) - 2:] == "gz":
+            cmd += ['--readFilesCommand', 'zcat']
+        cmd = ' '.join(cmd)
+        if self.STAR_param:
+            cmd += (" " + self.STAR_param)
+        Step_mapping.STAR.logger.info(cmd)
+        subprocess.check_call(cmd, shell=True)
 
-    @log
+    @add_log
     def picard(self):
-        self.refFlat, self.gtf = glob_genomeDir(self.genomeDir)
+        self.refFlat, _gtf = glob_genomeDir(self.genomeDir)
         self.picard_region_log = f'{self.outdir}/{self.sample}_region.log'
         cmd = [
             'picard',
@@ -162,9 +189,11 @@ class Step_mapping():
             (self.refFlat),
             'STRAND=NONE',
             'VALIDATION_STRINGENCY=SILENT']
-        res = subprocess.run(cmd, stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
+        cmd_str = ' '.join(cmd)
+        Step_mapping.picard.logger.info(cmd_str)
+        subprocess.check_call(cmd)
 
-    @log
+    @add_log
     def run(self):
         self.STAR()
         self.picard()
@@ -172,6 +201,8 @@ class Step_mapping():
             self.ribo()
         self.format_stat()
         self.report()
+        if not self.sort_BAM:
+            self.sort_bam()
     
     def report(self):
         t = reporter(
@@ -183,6 +214,18 @@ class Step_mapping():
             plot=self.plot)
         t.get_report()
 
+    @add_log
+    def sort_bam(self):
+        cmd = f'samtools sort {self.unsort_STAR_bam} -o {self.STAR_bam}'
+        Step_mapping.sort_bam.logger.info(cmd)
+        subprocess.check_call(cmd, shell=True)
+
+    @add_log
+    def index_bam(self):
+        cmd = f"samtools index {self.STAR_bam}"
+        Step_mapping.index_bam.logger.info(cmd)
+        subprocess.check_call(cmd, shell=True)
+
 
 def STAR(args):
     mapping = Step_mapping(
@@ -192,20 +235,29 @@ def STAR(args):
         args.thread,
         args.fq, 
         args.genomeDir, 
-        args.out_unmapped, 
-        args.debug,
-        args.outFilterMatchNmin)
+        out_unmapped=args.out_unmapped, 
+        debug=args.debug,
+        outFilterMatchNmin=args.outFilterMatchNmin,
+        STAR_param=args.STAR_param,
+        sort_BAM=True,
+        outFilterMultimapNmax=args.outFilterMultimapNmax,
+        STAR_index=args.STAR_index,
+        refFlat=args.refFlat,
+        consensus_fq=args.consensus_fq
+        )
     mapping.run()
 
 
 def get_opts_STAR(parser, sub_program):
-    if sub_program:
-        parser.add_argument('--fq', required=True)
-        parser.add_argument('--outdir', help='output dir', required=True)
-        parser.add_argument('--sample', help='sample name', required=True)
-        parser.add_argument('--thread', default=1)
-        parser.add_argument('--assay', help='assay', required=True)
-        parser.add_argument('--debug', help='debug', action='store_true')
     parser.add_argument('--outFilterMatchNmin', help='STAR outFilterMatchNmin', default=0)
     parser.add_argument('--out_unmapped', help='out_unmapped', action='store_true')
-    parser.add_argument('--genomeDir', help='genome directory', required=True)
+    parser.add_argument('--genomeDir', help='genome directory')
+    parser.add_argument('--STAR_param', help='STAR parameters', default="")
+    parser.add_argument('--STAR_index', help='STAR index directory')
+    parser.add_argument('--refFlat', help='refFlat file path')
+    parser.add_argument('--outFilterMultimapNmax', help='STAR outFilterMultimapNmax', default=1)
+    parser.add_argument("--consensus_fq", action='store_true', help="input fastq is umi consensus")
+    parser.add_argument('--starMem', help='starMem', default=30)
+    if sub_program:
+        parser.add_argument('--fq', required=True)
+        parser = s_common(parser)
