@@ -1,6 +1,3 @@
-#!/bin/env python
-# coding=utf8
-
 import os
 import sys
 import re
@@ -8,12 +5,17 @@ import subprocess
 import logging
 from itertools import islice
 import pandas as pd
+import pysam
 
-from celescope.tools.utils import format_number, log
+from celescope.tools.utils import *
 from celescope.tools.report import reporter
+from celescope.tools.Fastq import Fastq
 
 
-def format_stat(cutadapt_log, samplename):
+ADAPT = ['polyT=A{18}', 'p5=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC']
+
+
+def format_stat(cutadapt_log, sample_name):
     fh = open(cutadapt_log, 'r')
     stat_file = os.path.dirname(cutadapt_log) + '/stat.txt'
     # Total reads processed:...Total written (filtered):
@@ -40,21 +42,41 @@ def format_stat(cutadapt_log, samplename):
     fh.close()
 
 
-@log
+@add_log
+def read_adapter_fasta(adapter_fasta):
+    '''
+    return ['adapter1=AAA','adapter2=BBB']
+    '''
+    adapter_args = []
+    if adapter_fasta and adapter_fasta != 'None':
+        with pysam.FastxFile(adapter_fasta) as fh:
+            for read in fh:
+                adapter_args.append(f'{read.name}={read.sequence}')
+    return adapter_args
+
+
+@add_log
 def cutadapt(args):
     # check dir
     if not os.path.exists(args.outdir):
         os.system('mkdir -p %s' % (args.outdir))
 
+    adapter_args = read_adapter_fasta(args.adapter_fasta)
+    adapter_args += ADAPT
+
     # run cutadapt
     adapt = []
-    for a in args.adapt:
+    for a in adapter_args:
         adapt.append('-a')
         adapt.append(a)
 
-    out_fq2 = args.outdir + '/' + args.sample + '_clean_2.fq.gz'
+    if not args.not_gzip:
+        suffix = ".gz"
+    else:
+        suffix = ""
+    out_fq2 = f'{args.outdir}/{args.sample}_clean_2.fq{suffix}'
     cmd = ['cutadapt'] + adapt + ['-n',
-                                  str(len(args.adapt)),
+                                  str(len(adapter_args)),
                                   '-j',
                                   str(args.thread),
                                   '-m',
@@ -62,6 +84,8 @@ def cutadapt(args):
                                   '--nextseq-trim=' + str(args.nextseq_trim),
                                   '--overlap',
                                   str(args.overlap),
+                                  '-l',
+                                  str(args.insert),
                                   '-o',
                                   out_fq2,
                                   args.fq]
@@ -71,6 +95,7 @@ def cutadapt(args):
         fh.write(res.stdout)
 
     format_stat(args.outdir + '/cutadapt.log', args.sample)
+
     t = reporter(
         name='cutadapt',
         assay=args.assay,
@@ -83,30 +108,15 @@ def cutadapt(args):
 
 
 def get_opts_cutadapt(parser, sub_program):
+    parser.add_argument('--adapter_fasta', help='addtional adapter fasta file')
+    parser.add_argument('--minimum_length',dest='minimum_length',help='minimum_length', default=20)
+    parser.add_argument('--nextseq_trim', help='nextseq_trim', default=20)
+    parser.add_argument('--overlap',help='minimum overlap length', default=10)
+    parser.add_argument('--insert', help="read2 insert length", default=150)
     if sub_program:
         parser.add_argument('--fq', help='fq file', required=True)
-        parser.add_argument('--outdir', help='output dir', required=True)
-        parser.add_argument('--sample', help='sample name', required=True)
-        parser.add_argument('--assay', help='assay', required=True)
-    parser.add_argument(
-        '--adapt',
-        action='append',
-        default=[
-            'polyT=A{18}',
-            'p5=AGATCGGAAGAGCACACGTCTGAACTCCAGTCAC',
-        ])
-    parser.add_argument(
-        '--minimum_length',
-        dest='minimum_length',
-        help='minimum_length, default=20',
-        default=20)
-    parser.add_argument(
-        '--nextseq-trim',
-        dest='nextseq_trim',
-        help='nextseq_trim, default=20',
-        default=20)
-    parser.add_argument(
-        '--overlap',
-        help='minimum overlap length',
-        default=10)
-    parser.add_argument('--thread', default=2)
+        parser.add_argument('--not_gzip', help="output fastq without gzip", action='store_true')
+        parser = s_common(parser)
+    return parser
+
+
