@@ -7,12 +7,10 @@ import logging
 import subprocess
 import glob
 import sys
-from celescope.tools.utils import format_number
-from celescope.tools.utils import glob_genomeDir
+import pysam
+from celescope.tools.utils import *
 from celescope.tools.report import reporter
 
-logger1 = logging.getLogger(__name__)
-
 
 def format_stat(log, samplename):
     #Assigned, Unassigned_NoFeatures, Unassigned_Ambiguity=(0, 0, 0)
@@ -47,14 +45,37 @@ def format_stat(log, samplename):
             stat_fh.write('%s: %s\n' % (t, s))
     fh.close()
 
+@add_log
+def add_tag(bam, gtf):
+    id_name = gene_convert(gtf)
+    samfile = pysam.AlignmentFile(bam, "rb")
+    header = samfile.header
+    new_bam = pysam.AlignmentFile(
+        bam + ".temp", "wb", header=header)
+    for read in samfile:
+        attr = read.query_name.split('_')
+        barcode = attr[0]
+        umi = attr[1]
+        read.set_tag(tag='CB', value=barcode, value_type='Z')
+        read.set_tag(tag='UB', value=umi, value_type='Z')
+        if read.has_tag('XT'):
+            gene_id = read.get_tag('XT')
+            gene_name = id_name[gene_id]
+            read.set_tag(tag='GN', value=gene_name, value_type='Z')
+            read.set_tag(tag='GX', value=gene_id, value_type='Z')
+        new_bam.write(read)
+    new_bam.close()
+    cmd = f'mv {bam}.temp {bam}'
+    subprocess.check_call(cmd, shell=True)
 
+@add_log
 def featureCounts(args):
-    """
-    """
-    logger1.info('featureCounts ...!')
 
     # check
-    refFlat, gtf = glob_genomeDir(args.genomeDir, logger1)
+    if args.genomeDir and args.genomeDir != "None":
+        _refFlat, gtf = glob_genomeDir(args.genomeDir)
+    else:
+        gtf = args.gtf
 
     # check dir
     if not os.path.exists(args.outdir):
@@ -62,33 +83,42 @@ def featureCounts(args):
 
     # run featureCounts
     outPrefix = args.outdir + '/' + args.sample
-    cmd = ['featureCounts', '-a', gtf, '-o', outPrefix, '-R', 'BAM',
-           '-T', str(args.thread), '-t', args.gtf_type, args.input]
-    logger1.info('%s' % (' '.join(cmd)))
+    cmd = [
+        'featureCounts', 
+        '-s', '1',
+        '-a', gtf, 
+        '-o', outPrefix, 
+        '-R', 'BAM',
+        '-T', str(args.thread), 
+        '-t', args.gtf_type, 
+        args.input,
+    ]
+    featureCounts.logger.info('%s' % (' '.join(cmd)))
     subprocess.check_call(cmd)
-    logger1.info('featureCounts done!')
 
     subprocess.check_call(['which', 'samtools'])
 
-    # sort by name:BC and umi
-    logger1.info('samtools sort ...!')
     bam_basename = os.path.basename(args.input)
+    bam = f'{args.outdir}/{bam_basename}.featureCounts.bam'
+
+    # add tag
+    add_tag(bam, gtf)
+
+    # sort by name:BC and umi
+    featureCounts.logger.info('samtools sort ...!')
     cmd = [
         'samtools',
         'sort',
         '-n',
         '-@',
-        '3',
+        str(args.thread),
         '-o',
         outPrefix +
         '_name_sorted.bam',
-        args.outdir +
-        '/' +
-        bam_basename +
-        '.featureCounts.bam']
-    logger1.info('%s' % (' '.join(cmd)))
+        bam]
+    featureCounts.logger.info('%s' % (' '.join(cmd)))
     subprocess.check_call(cmd)
-    logger1.info('samtools sort done!')
+    featureCounts.logger.info('samtools sort done.')
 
     format_stat(args.outdir + '/' + args.sample + '.summary', args.sample)
     t = reporter(
@@ -103,16 +133,14 @@ def featureCounts(args):
 
 
 def get_opts_featureCounts(parser, sub_program):
-
+    parser.add_argument('--gtf', help='gtf file path')
     parser.add_argument(
         '--gtf_type',
         help='Specify feature type in GTF annotation',
         default='exon')
+    parser.add_argument('--genomeDir')
     if sub_program:
-        parser.add_argument('--genomeDir', required=True)
-        parser.add_argument('--thread', default=1)
+        parser = s_common(parser)
         parser.add_argument('--input', required=True)
-        #parser.add_argument('--format', default='BAM')
-        parser.add_argument('--outdir', help='output dir', required=True)
-        parser.add_argument('--sample', help='sample name', required=True)
-        parser.add_argument('--assay', help='assay', required=True)
+    return parser
+
