@@ -10,6 +10,7 @@ import subprocess
 import time
 import argparse
 import pysam
+import importlib
 from datetime import timedelta
 from collections import defaultdict
 from functools import wraps
@@ -24,7 +25,7 @@ from celescope.tools.__init__ import __PATTERN_DICT__
 tools_dir = os.path.dirname(celescope.tools.__file__)
 
 
-def log(func):
+def add_log(func):
     '''
     return logger.
     logging start and done.
@@ -61,22 +62,24 @@ def arg_str(arg, arg_name):
     return ''
 
 
-def read_barcode_file(match_dir):
+def read_barcode_file(match_dir, return_file=False):
     '''
     multi version compatible
     '''
     match_barcode_file1 = glob.glob(
-        f"{match_dir}/*count/*_cellbarcode.tsv")
+        f"{match_dir}/*count*/*_cellbarcode.tsv")
     match_barcode_file2 = glob.glob(
-        f"{match_dir}/*count/*matrix_10X/*_cellbarcode.tsv")
+        f"{match_dir}/*count*/*matrix_10X/*_cellbarcode.tsv")
     match_barcode_file3 = glob.glob(
-        f"{match_dir}/*count/*matrix_10X/*barcodes.tsv")
+        f"{match_dir}/*count*/*matrix_10X/*barcodes.tsv")
     match_barcode_file = (
         match_barcode_file1 +
         match_barcode_file2 +
         match_barcode_file3)[0]
     match_barcode, cell_total = read_one_col(match_barcode_file)
     match_barcode = set(match_barcode)
+    if return_file:
+        return match_barcode, cell_total, match_barcode_file
     return match_barcode, cell_total
 
 
@@ -156,13 +159,21 @@ def link_data(outdir, fq_dict):
             fh.write('ln -sf %s %s\n' % (arr[1], s + '_2.fq.gz'))
 
 
+def generic_open(file_name, mode='rt'):
+    if file_name.endswith('.gz'):
+        file_obj = gzip.open(file_name, mode)
+    else:
+        file_obj = open(file_name, mode)
+    return file_obj
+
+
 def gene_convert(gtf_file):
 
     gene_id_pattern = re.compile(r'gene_id "(\S+)";')
     gene_name_pattern = re.compile(r'gene_name "(\S+)"')
     id_name = {}
     c = Counter()
-    with open(gtf_file) as f:
+    with generic_open(gtf_file) as f:
         for line in f:
             if not line.strip():
                 continue
@@ -184,7 +195,7 @@ def gene_convert(gtf_file):
     return id_name
 
 
-@log
+@add_log
 def process_read(
     read2_file, pattern_dict, barcode_dict, linker_dict,
     barcode_length, linker_length):
@@ -326,6 +337,8 @@ def gen_stat(df, stat_file):
         percent = count / row['total_count']
         value = f'{format_number(count)}({round(percent * 100, 2)}%)'
         return value
+
+    df.loc[:,'value'] = df.loc[:,'count']
     df.loc[~df['total_count'].isna(), 'value'] = df.loc[~df['total_count'].isna(), :].apply(
         lambda row: add_percent(row), axis=1
     )
@@ -344,8 +357,10 @@ def get_fq(library_id, library_path):
         pattern2_1 = library_path + '/' + library_id + '*' + '_2.fq.gz'
         pattern2_2 = f'{library_path}/*{library_id}*R2.fastq.gz'
         pattern2_3 = f'{library_path}/*{library_id}*R2_001.fastq.gz'
-        fq1 = ",".join(glob.glob(pattern1_1) + glob.glob(pattern1_2) + glob.glob(pattern1_3))
-        fq2 = ",".join(glob.glob(pattern2_1) + glob.glob(pattern2_2) + glob.glob(pattern2_3))
+        fq1_list = sorted(glob.glob(pattern1_1) + glob.glob(pattern1_2) + glob.glob(pattern1_3))
+        fq2_list = sorted(glob.glob(pattern2_1) + glob.glob(pattern2_2) + glob.glob(pattern2_3))
+        fq1 = ",".join(fq1_list)
+        fq2 = ",".join(fq2_list)
         if len(fq1) == 0:
             sys.exit('Invalid fastq name pattern!')
     except IndexError as e:
@@ -426,7 +441,20 @@ def format_number(number: int) -> str:
     return format(number, ",")
 
 
-@log
+def format_metrics(metrics: dict):
+    for key in metrics:
+        value = metrics[key]
+        metrics[key] = format_number(value)
+        if isinstance(value, float):
+            metrics[key] = round(value, 2)
+
+
+def format_ratios(ratios: dict):
+    for key in ratios:
+        ratios[key] = round(ratios[key] * 100, 2)
+
+
+@add_log
 def glob_genomeDir(genomeDir, fa=False):
     refFlat = glob.glob(genomeDir + "/*.refFlat")
     if (len(refFlat) > 1):
@@ -578,7 +606,7 @@ def genDict(dim=3, valType=int):
         return defaultdict(lambda: genDict(dim - 1, valType=valType))
 
 
-@log
+@add_log
 def downsample(bam, barcodes, percent):
     """
     calculate median_geneNum and saturation based on a given percent of reads
@@ -685,30 +713,32 @@ def report_prepare(outdir, **kwargs):
         json.dump(data, fh)
 
 
-def parse_vcf(vcf_file, cols=['chrom', 'pos', 'alleles'], infos=['DP', 'GENE', 'CELL']):
+def parse_vcf(vcf_file, cols=['chrom', 'pos', 'alleles'], infos=['VID','CID']):
     vcf = pysam.VariantFile(vcf_file)
-    df = pd.DataFrame(columns=[col.upper() for col in cols] + infos + ['GT'])
+    df = pd.DataFrame(columns=[col.capitalize() for col in cols] + infos)
     rec_dict = {}
     for rec in vcf.fetch():
 
         for col in cols:
-            rec_dict[col.upper()] = getattr(rec, col)
+            rec_dict[col.capitalize()] = getattr(rec, col)
             if col == 'alleles':
-                rec_dict['ALLELES'] = '-'.join(rec_dict['ALLELES'])
+                rec_dict['Alleles'] = '-'.join(rec_dict['Alleles'])
                 
         for info in infos:
             rec_dict[info] = rec.info[info]
 
+        '''
         rec_dict['GT'] = [s['GT'] for s in rec.samples.values()][0]
         rec_dict['GT'] = [str(item) for item in rec_dict['GT']]
         rec_dict['GT'] = '/'.join(rec_dict['GT'])
+        '''
 
         df = df.append(pd.Series(rec_dict),ignore_index=True)
     return df
 
 
 def parse_annovar(annovar_file):
-    df = pd.DataFrame(columns=['mRNA', 'protein', 'cosmic'])
+    df = pd.DataFrame(columns=['Gene','mRNA', 'Protein', 'COSMIC'])
     with open(annovar_file, 'rt') as f:
         index = 0
         for line in f:
@@ -716,6 +746,7 @@ def parse_annovar(annovar_file):
             if index == 1:
                 continue
             attrs = line.split('\t')
+            gene = attrs[6]
             func = attrs[5]
             if func == 'exonic':
                 changes = attrs[9]
@@ -742,8 +773,9 @@ def parse_annovar(annovar_file):
             mRNA = combine[0]
             protein = combine[1]
             df = df.append({
-                'MRNA': mRNA,
-                'PROTEIN': protein,
+                'Gene': gene,
+                'mRNA': mRNA,
+                'Protein': protein,
                 'COSMIC': cosmic,
             }, ignore_index=True)
     return df
@@ -763,7 +795,7 @@ def parse_match_dir(match_dir):
     return match_dict
 
 
-@log
+@add_log
 def STAR_util(
     sample,
     outdir,
@@ -773,6 +805,7 @@ def STAR_util(
     outFilterMatchNmin=35,
     out_unmapped=False,
     outFilterMultimapNmax=1,
+    outBAMsortingBinsN=50,
 ):
 
     # check dir
@@ -790,7 +823,9 @@ def STAR_util(
  --runThreadN {runThreadN}\
  --outFilterMatchNmin {outFilterMatchNmin}\
  --outFileNamePrefix {out_prefix}\
- --outFilterMultimapNmax {outFilterMultimapNmax} "
+ --outFilterMultimapNmax {outFilterMultimapNmax}\
+ --outBAMsortingBinsN {outBAMsortingBinsN}\
+ --limitBAMsortRAM 100000000000 "
     
     if out_unmapped:
         cmd += ' --outReadsUnmapped Fastx '
@@ -810,7 +845,7 @@ def get_scope_bc(bctype):
     whitelist_f = f'{root_path}/data/chemistry/{bctype}/bclist'
     return linker_f, whitelist_f
 
-@log
+@add_log
 def parse_pattern(pattern):
     # 解析接头结构，返回接头结构字典
     # key: 字母表示的接头, value: 碱基区间列表
@@ -831,7 +866,35 @@ def parse_pattern(pattern):
     return pattern_dict
     
 
+def fastq_line(name, seq, qual):
+    return f'@{name}\n{seq}\n+\n{qual}\n'
 
 
+def s_common(parser):
+    """subparser common arguments
+    """
+    parser.add_argument('--outdir', help='output dir', required=True)
+    parser.add_argument('--assay', help='assay', required=True)
+    parser.add_argument('--sample', help='sample name', required=True)
+    parser.add_argument('--thread', default=4)
+    parser.add_argument('--debug', help='debug', action='store_true')
+    return parser
 
 
+def find_assay_init(assay):
+    init_module = importlib.import_module(f"celescope.{assay}.__init__")
+    return init_module
+
+
+def find_step_module(assay, step):
+    init_module = find_assay_init(assay)
+    try:
+        step_module = importlib.import_module(f"celescope.{assay}.{step}")
+    except ModuleNotFoundError as error:
+        try:
+            step_module = importlib.import_module(f"celescope.tools.{step}")
+        except ModuleNotFoundError as error:
+            module_path = init_module.IMPORT_DICT[step]
+            step_module = importlib.import_module(f"{module_path}.{step}")
+
+    return step_module
\ No newline at end of file
